library basis2d;

import 'vectors/abstract_vector.dart';
import 'vectors/double_vector.dart';

abstract class Abstract2DBasis<N extends num> {
	Abstract2DVector<N> _origin;
	Abstract2DVector<N> _e1;
	Abstract2DVector<N> _e2;

	N screenX( N basisX, N basisY );
	N screenY( N basisX, N basisY );
	N basisX( N screenX, N screenY );
	N basisY( N screenX, N screenY );

	Abstract2DVector<N> get origin => _origin;
	Abstract2DVector<N> get e1 => _e1;
	Abstract2DVector<N> get e2 => _e2;
}


class CustomDouble2DBasis extends Abstract2DBasis<double> {
	CustomDouble2DBasis( double x0, double y0, double ex1, double ey1, double ex2, double ey2 ) {
		_origin = new CustomDouble2DVector( x0, y0 );
		_e1 = new CustomDouble2DVector( ex1, ey1 );
		_e2 = new CustomDouble2DVector( ex2, ey2 );
	}

	double screenX( double basisX, double basisY ) => basisX * _e1.x + _origin.x;
	double screenY( double basisX, double basisY ) => basisY * _e2.y + _origin.y;
	double basisX( double screenX, double screenY ) => (screenX - _origin.x ) / _e1.x;
	double basisY( double screenX, double screenY ) => (screenY - _origin.y ) / _e2.y;
}


class OrthogonalDouble2DBasis extends Abstract2DBasis<double> {
	OrthogonalDouble2DBasis( double x0, double y0, double xLength, double yLength ) {
		_origin = new CustomDouble2DVector( x0, y0 );
		_e1 = new HorizontalDouble2DVector( xLength );
		_e2 = new VerticalDouble2DVector( yLength );
	}

	OrthogonalDouble2DBasis.equalLength( double x0, double y0, double length ) : this( x0, y0, length, length );

	double screenX( double basisX, double basisY ) => basisX * _e1.x + _origin.x;
	double screenY( double basisX, double basisY ) => basisY * _e2.y + _origin.y;
	double basisX( double screenX, double screenY ) => ( screenX - _origin.x ) / _e1.x;
	double basisY( double screenX, double screenY ) => ( screenY - _origin.y ) / _e2.y;
}