library plane2d;

import 'shape.dart';
import '../vectors/abstract_vector.dart';
import '../vectors/double_vector.dart';

abstract class AbstractHalfPlane<N extends num> extends Abstract2DShape<N> {
	N get a;
	N get b;
	N get c;

	set leftX( N value ) {}
	set rightX( N value ) {}
	set topY( N value ) {}
	set bottomY( N value ) {}
}


abstract class AbstractDoubleHalfPlane extends AbstractHalfPlane<double> {
	Abstract2DVector<double> get position => NullDouble2DVector.instance;
	Abstract2DVector<double> get size => NullDouble2DVector.instance;
}


abstract class OrthogonalDoubleHalfPlane extends AbstractDoubleHalfPlane {
	double _distance;

	OrthogonalDoubleHalfPlane( this._distance );

	set distance( double value ) => _distance = value;
}


class LeftOrthogonalDoubleHalfPlane extends OrthogonalDoubleHalfPlane {
	LeftOrthogonalDoubleHalfPlane( double distance ) : super( distance );

	double get a => 1.0;
	double get b => 0.0;
	double get c => _distance;
}

class RightOrthogonalDoubleHalfPlane extends OrthogonalDoubleHalfPlane {
	RightOrthogonalDoubleHalfPlane( double distance ) : super( distance );

	double get a => -1.0;
	double get b => 0.0;
	double get c => _distance;
}

class TopOrthogonalDoubleHalfPlane extends OrthogonalDoubleHalfPlane {
	TopOrthogonalDoubleHalfPlane( double distance ) : super( distance );

	double get a => 0.0;
	double get b => 1.0;
	double get c => _distance;
}

class BottomOrthogonalDoubleHalfPlane extends OrthogonalDoubleHalfPlane {
	BottomOrthogonalDoubleHalfPlane( double distance ) : super( distance );

	double get a => 0.0;
	double get b => -1.0;
	double get c => _distance;
}


class CustomDoubleHalfPlane extends AbstractDoubleHalfPlane {
	double _a, _b, _c;

	double get a => _a;
	double get b => _b;
	double get c => _c;

	void set( double a, double b, double c ) {
		assert( a != 0 || b != 0.0 );
		_a = a;
		_b = b;
		_c = c;
	}
}