library matrix_2d_space_partition_system_for_double_shapes;

import "dart:math";
import 'abstract_system.dart';
import '../shapes/shape.dart';
import '../vectors/abstract_vector.dart';
import '../vectors/double_vector.dart';


class Matrix2DSpacePartitioningSystemForDoubleShapes extends Abstract2DSpacePartitioningSystem<double> {
	final CustomDouble2DVector _origin = new CustomDouble2DVector(0.0, 0.0);
	final Map<Abstract2DShape<double>, Custom2DSpacePartitioningSystemMatrix<Abstract2DShape<double>>> shapes = new Map<Abstract2DShape<double>, Custom2DSpacePartitioningSystemMatrix<Abstract2DShape<double>>>();

	Custom2DSpacePartitioningSystemMatrix _root;

	Matrix2DSpacePartitioningSystemForDoubleShapes( double x, double y, int cellSize, int matrixSize, int level ) {
		_origin.set( x, y );
		_root = new Custom2DSpacePartitioningSystemMatrix<Abstract2DShape<double>>( null, 0, 0, pow( matrixSize, level ), level );
	}

	Custom2DSpacePartitioningSystemMatrix findMatrix( double x, double y ) {
		int xPos = ( ( x - _origin.x ) / _root._cellSize ).ceil();
		int yPos = ( ( y - _origin.y ) / _root._cellSize ).ceil();
		int matrixSize = _root._cellSize * _root._cellsQuantity;
		if( xPos < 0 || yPos < 0 || xPos >= matrixSize || yPos >= matrixSize ) {
			int newLevel = _root._level + 1;
			Custom2DSpacePartitioningSystemMatrix _newRoot
					= new Custom2DSpacePartitioningSystemMatrix<Abstract2DShape<double>>( null, 0, 0, pow( matrixSize, newLevel ), newLevel );
		}
		return _root.findChildMatrix( xPos, yPos );
	}

	void moveShape(Abstract2DShape<double> shape, Abstract2DVector<double> newPosition) {

	}

	void insertShape(Abstract2DShape<double> shape) {

	}
}


class Custom2DSpacePartitioningSystemMatrix<O> {
	Custom2DSpacePartitioningSystemMatrix _parent;

	int _cellsQuantity, _cellSize, _matrixSize, _x, _y, _level;

	List<Set<O>> _listsArray;
	List<Custom2DSpacePartitioningSystemMatrix> _matrixArray;
	List<Custom2DSpacePartitioningSystemMatrix> _nearbyMatrices;

	Custom2DSpacePartitioningSystemMatrix( this._parent, this._x, this._y, this._level ) {
		_matrixSize = _parent._cellSize;
		_cellsQuantity = _parent._cellsQuantity;
		_cellSize = _matrixSize ~/ _cellsQuantity;
		_nearbyMatrices = new List<Custom2DSpacePartitioningSystemMatrix<O>>( 9 );

		if ( _level == 1 ) {
			_listsArray = new List<Set<O>>( _cellsQuantity * _cellsQuantity );
		} else {
			_matrixArray = new List<Custom2DSpacePartitioningSystemMatrix<O>>( _cellsQuantity * _cellsQuantity );
		}
	}

	Custom2DSpacePartitioningSystemMatrix<O> findChildMatrix( int x, int y ) {
		if( _level == 1 ) return this;
		int xPos = ( x - _x ) ~/ _cellSize;
		int yPos = ( y - _y ) ~/ _cellSize;
		int pos = xPos ~/ _cellSize + ( yPos ~/ _cellSize ) * _cellSize;
		Custom2DSpacePartitioningSystemMatrix<O> childMatrix = _matrixArray[ pos ];
		if( childMatrix == null ) {
			childMatrix = new Custom2DSpacePartitioningSystemMatrix<O>( this, _x + _cellSize * xPos, _y + _cellSize * yPos, _cellsQuantity, _level - 1 );
			_matrixArray[ pos ] = childMatrix;
		}
		return childMatrix;
	}
}
